#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'configgame.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGGAME_H
#define UI_CONFIGGAME_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QSlider>
#include <QtGui/QSpacerItem>
#include <QtGui/QSpinBox>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ConfigGame
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *kcfg_PrintMulti;
    QSpacerItem *verticalSpacer;
    QCheckBox *kcfg_ShowHighlights;
    QCheckBox *kcfg_ShowErrors;
    QCheckBox *kcfg_ShowHighlights3D;
    QSpacerItem *spacer_3;
    QHBoxLayout *horizontalLayout_5;
    QSpinBox *kcfg_MathdokuSize;
    QLabel *label_4;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout;
    QSlider *kcfg_OverallSize3D;
    QLabel *label_6;
    QHBoxLayout *horizontalLayout_1;
    QSlider *kcfg_SelectionSize3D;
    QLabel *label;
    QHBoxLayout *horizontalLayout_2;
    QSlider *kcfg_HighlightedSize3D;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_4;
    QSlider *kcfg_OuterCellSize3D;
    QLabel *label_5;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *kcfg_DarkenOuterCells3D;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *ConfigGame)
    {
        if (ConfigGame->objectName().isEmpty())
            ConfigGame->setObjectName(QString::fromUtf8("ConfigGame"));
        ConfigGame->resize(462, 521);
        verticalLayout = new QVBoxLayout(ConfigGame);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        kcfg_PrintMulti = new QCheckBox(ConfigGame);
        kcfg_PrintMulti->setObjectName(QString::fromUtf8("kcfg_PrintMulti"));

        verticalLayout->addWidget(kcfg_PrintMulti);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout->addItem(verticalSpacer);

        kcfg_ShowHighlights = new QCheckBox(ConfigGame);
        kcfg_ShowHighlights->setObjectName(QString::fromUtf8("kcfg_ShowHighlights"));

        verticalLayout->addWidget(kcfg_ShowHighlights);

        kcfg_ShowErrors = new QCheckBox(ConfigGame);
        kcfg_ShowErrors->setObjectName(QString::fromUtf8("kcfg_ShowErrors"));

        verticalLayout->addWidget(kcfg_ShowErrors);

        kcfg_ShowHighlights3D = new QCheckBox(ConfigGame);
        kcfg_ShowHighlights3D->setObjectName(QString::fromUtf8("kcfg_ShowHighlights3D"));

        verticalLayout->addWidget(kcfg_ShowHighlights3D);

        spacer_3 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout->addItem(spacer_3);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        horizontalLayout_5->setContentsMargins(-1, 0, -1, 0);
        kcfg_MathdokuSize = new QSpinBox(ConfigGame);
        kcfg_MathdokuSize->setObjectName(QString::fromUtf8("kcfg_MathdokuSize"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(200);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(kcfg_MathdokuSize->sizePolicy().hasHeightForWidth());
        kcfg_MathdokuSize->setSizePolicy(sizePolicy);
        kcfg_MathdokuSize->setMinimumSize(QSize(84, 0));
        kcfg_MathdokuSize->setMinimum(3);
        kcfg_MathdokuSize->setMaximum(9);
        kcfg_MathdokuSize->setValue(6);

        horizontalLayout_5->addWidget(kcfg_MathdokuSize);

        label_4 = new QLabel(ConfigGame);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy1);

        horizontalLayout_5->addWidget(label_4);


        verticalLayout->addLayout(horizontalLayout_5);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout->addItem(verticalSpacer_2);

        label_2 = new QLabel(ConfigGame);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QFont font;
        font.setPointSize(13);
        label_2->setFont(font);

        verticalLayout->addWidget(label_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        kcfg_OverallSize3D = new QSlider(ConfigGame);
        kcfg_OverallSize3D->setObjectName(QString::fromUtf8("kcfg_OverallSize3D"));
        sizePolicy.setHeightForWidth(kcfg_OverallSize3D->sizePolicy().hasHeightForWidth());
        kcfg_OverallSize3D->setSizePolicy(sizePolicy);
        kcfg_OverallSize3D->setMinimum(7);
        kcfg_OverallSize3D->setMaximum(13);
        kcfg_OverallSize3D->setSingleStep(1);
        kcfg_OverallSize3D->setPageStep(1);
        kcfg_OverallSize3D->setValue(10);
        kcfg_OverallSize3D->setSliderPosition(10);
        kcfg_OverallSize3D->setOrientation(Qt::Horizontal);
        kcfg_OverallSize3D->setInvertedAppearance(false);
        kcfg_OverallSize3D->setTickPosition(QSlider::TicksBelow);
        kcfg_OverallSize3D->setTickInterval(1);

        horizontalLayout->addWidget(kcfg_OverallSize3D);

        label_6 = new QLabel(ConfigGame);
        label_6->setObjectName(QString::fromUtf8("label_6"));
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(label_6);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_1 = new QHBoxLayout();
        horizontalLayout_1->setObjectName(QString::fromUtf8("horizontalLayout_1"));
        kcfg_SelectionSize3D = new QSlider(ConfigGame);
        kcfg_SelectionSize3D->setObjectName(QString::fromUtf8("kcfg_SelectionSize3D"));
        sizePolicy.setHeightForWidth(kcfg_SelectionSize3D->sizePolicy().hasHeightForWidth());
        kcfg_SelectionSize3D->setSizePolicy(sizePolicy);
        kcfg_SelectionSize3D->setMinimum(7);
        kcfg_SelectionSize3D->setMaximum(13);
        kcfg_SelectionSize3D->setSingleStep(1);
        kcfg_SelectionSize3D->setPageStep(1);
        kcfg_SelectionSize3D->setValue(7);
        kcfg_SelectionSize3D->setSliderPosition(7);
        kcfg_SelectionSize3D->setOrientation(Qt::Horizontal);
        kcfg_SelectionSize3D->setInvertedAppearance(false);
        kcfg_SelectionSize3D->setTickPosition(QSlider::TicksBelow);
        kcfg_SelectionSize3D->setTickInterval(1);

        horizontalLayout_1->addWidget(kcfg_SelectionSize3D);

        label = new QLabel(ConfigGame);
        label->setObjectName(QString::fromUtf8("label"));
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        horizontalLayout_1->addWidget(label);


        verticalLayout->addLayout(horizontalLayout_1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        kcfg_HighlightedSize3D = new QSlider(ConfigGame);
        kcfg_HighlightedSize3D->setObjectName(QString::fromUtf8("kcfg_HighlightedSize3D"));
        sizePolicy.setHeightForWidth(kcfg_HighlightedSize3D->sizePolicy().hasHeightForWidth());
        kcfg_HighlightedSize3D->setSizePolicy(sizePolicy);
        kcfg_HighlightedSize3D->setMinimum(7);
        kcfg_HighlightedSize3D->setMaximum(13);
        kcfg_HighlightedSize3D->setPageStep(1);
        kcfg_HighlightedSize3D->setValue(7);
        kcfg_HighlightedSize3D->setOrientation(Qt::Horizontal);
        kcfg_HighlightedSize3D->setTickPosition(QSlider::TicksBelow);
        kcfg_HighlightedSize3D->setTickInterval(1);

        horizontalLayout_2->addWidget(kcfg_HighlightedSize3D);

        label_3 = new QLabel(ConfigGame);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(label_3);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        kcfg_OuterCellSize3D = new QSlider(ConfigGame);
        kcfg_OuterCellSize3D->setObjectName(QString::fromUtf8("kcfg_OuterCellSize3D"));
        sizePolicy.setHeightForWidth(kcfg_OuterCellSize3D->sizePolicy().hasHeightForWidth());
        kcfg_OuterCellSize3D->setSizePolicy(sizePolicy);
        kcfg_OuterCellSize3D->setMinimum(4);
        kcfg_OuterCellSize3D->setMaximum(10);
        kcfg_OuterCellSize3D->setPageStep(1);
        kcfg_OuterCellSize3D->setValue(4);
        kcfg_OuterCellSize3D->setOrientation(Qt::Horizontal);
        kcfg_OuterCellSize3D->setTickPosition(QSlider::TicksBelow);
        kcfg_OuterCellSize3D->setTickInterval(1);

        horizontalLayout_4->addWidget(kcfg_OuterCellSize3D);

        label_5 = new QLabel(ConfigGame);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        sizePolicy1.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy1);

        horizontalLayout_4->addWidget(label_5);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        horizontalLayout_3->setContentsMargins(-1, 5, -1, -1);
        kcfg_DarkenOuterCells3D = new QCheckBox(ConfigGame);
        kcfg_DarkenOuterCells3D->setObjectName(QString::fromUtf8("kcfg_DarkenOuterCells3D"));

        horizontalLayout_3->addWidget(kcfg_DarkenOuterCells3D);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_3);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(spacerItem);

#ifndef UI_QT_NO_SHORTCUT
        label_6->setBuddy(kcfg_OverallSize3D);
        label->setBuddy(kcfg_SelectionSize3D);
        label_3->setBuddy(kcfg_HighlightedSize3D);
        label_5->setBuddy(kcfg_OuterCellSize3D);
#endif // QT_NO_SHORTCUT
        QWidget::setTabOrder(kcfg_ShowHighlights, kcfg_ShowErrors);
        QWidget::setTabOrder(kcfg_ShowErrors, kcfg_ShowHighlights3D);
        QWidget::setTabOrder(kcfg_ShowHighlights3D, kcfg_SelectionSize3D);
        QWidget::setTabOrder(kcfg_SelectionSize3D, kcfg_HighlightedSize3D);
        QWidget::setTabOrder(kcfg_HighlightedSize3D, kcfg_OuterCellSize3D);
        QWidget::setTabOrder(kcfg_OuterCellSize3D, kcfg_DarkenOuterCells3D);

        retranslateUi(ConfigGame);

        QMetaObject::connectSlotsByName(ConfigGame);
    } // setupUi

    void retranslateUi(QWidget *ConfigGame)
    {
#ifndef UI_QT_NO_TOOLTIP
        kcfg_PrintMulti->setToolTip(QString());
#endif // QT_NO_TOOLTIP
        kcfg_PrintMulti->setText(tr2i18n("When printing, try to fit more than one puzzle on a page", 0));
#ifndef UI_QT_NO_TOOLTIP
        kcfg_ShowHighlights->setToolTip(tr2i18n("Highlight the row, column and block that contain the selected cell", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        kcfg_ShowHighlights->setWhatsThis(tr2i18n("Highlight the row, column and block that contain the selected cell. Blocks may be square or irregular (as in jigsaw-type or XSudoku puzzles).", 0));
#endif // QT_NO_WHATSTHIS
        kcfg_ShowHighlights->setText(tr2i18n("Show highlights in 2-D puzzles", 0));
#ifndef UI_QT_NO_TOOLTIP
        kcfg_ShowErrors->setToolTip(tr2i18n("Highlight in red any cells whose values break the Sudoku rules.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        kcfg_ShowErrors->setWhatsThis(tr2i18n("Highlight in red any cells whose values break the Sudoku rules.", 0));
#endif // QT_NO_WHATSTHIS
        kcfg_ShowErrors->setText(tr2i18n("Show errors", 0));
#ifndef UI_QT_NO_TOOLTIP
        kcfg_ShowHighlights3D->setToolTip(tr2i18n("Highlight the planes (blocks) that contain the selected cell.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        kcfg_ShowHighlights3D->setWhatsThis(tr2i18n("Highlight the planes (blocks) that contain the selected cell.", 0));
#endif // QT_NO_WHATSTHIS
        kcfg_ShowHighlights3D->setText(tr2i18n("Show highlights in 3-D puzzles", 0));
        label_4->setText(tr2i18n("Mathdoku puzzle size (3-9)", 0));
        label_2->setText(tr2i18n("Settings for 3-D Puzzles Only", 0));
#ifndef UI_QT_NO_TOOLTIP
        kcfg_OverallSize3D->setToolTip(tr2i18n("Set the relative size of 3-D cells and the spaces between them.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        kcfg_OverallSize3D->setWhatsThis(tr2i18n("Set the relative size of 3-D cells and the spaces between them.", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_TOOLTIP
        label_6->setToolTip(tr2i18n("Set the relative size of 3-D cells and the spaces between them.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        label_6->setWhatsThis(tr2i18n("Set the relative size of 3-D cells and the spaces between them.", 0));
#endif // QT_NO_WHATSTHIS
        label_6->setText(tr2i18n("Overall size of 3-D cells", 0));
#ifndef UI_QT_NO_TOOLTIP
        kcfg_SelectionSize3D->setToolTip(tr2i18n("Shrink or expand a cell by up to 30% when it is selected.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        kcfg_SelectionSize3D->setWhatsThis(tr2i18n("Shrink or expand a cell by up to 30% when it is selected.", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_TOOLTIP
        label->setToolTip(tr2i18n("Shrink or expand a cell by up to 30% when it is selected.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        label->setWhatsThis(tr2i18n("Shrink or expand a cell by up to 30% when it is selected.", 0));
#endif // QT_NO_WHATSTHIS
        label->setText(tr2i18n("Size of selected 3-D cell", 0));
#ifndef UI_QT_NO_TOOLTIP
        kcfg_HighlightedSize3D->setToolTip(tr2i18n("Shrink or expand cells by up to 30% when they are highlighted.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        kcfg_HighlightedSize3D->setWhatsThis(tr2i18n("Shrink or expand cells by up to 30% when they are highlighted.", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_TOOLTIP
        label_3->setToolTip(tr2i18n("Shrink or expand cells by up to 30% when they are highlighted.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        label_3->setWhatsThis(tr2i18n("Shrink or expand cells by up to 30% when they are highlighted.", 0));
#endif // QT_NO_WHATSTHIS
        label_3->setText(tr2i18n("Size of highlighted 3-D cells", 0));
#ifndef UI_QT_NO_TOOLTIP
        kcfg_OuterCellSize3D->setToolTip(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        kcfg_OuterCellSize3D->setWhatsThis(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell. This is useful if a 3-D puzzle has more than one cube.", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_TOOLTIP
        label_5->setToolTip(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        label_5->setWhatsThis(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell. This is useful if a 3-D puzzle has more than one cube.", 0));
#endif // QT_NO_WHATSTHIS
        label_5->setText(tr2i18n("Size of 3-D cells outside the selected volume", 0));
#ifndef UI_QT_NO_TOOLTIP
        kcfg_DarkenOuterCells3D->setToolTip(tr2i18n("Darken cells if they are not in an NxNxN cube that contains the selected cell.", 0));
#endif // QT_NO_TOOLTIP
#ifndef UI_QT_NO_WHATSTHIS
        kcfg_DarkenOuterCells3D->setWhatsThis(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell. This is useful if a 3-D puzzle has more than one cube.", 0));
#endif // QT_NO_WHATSTHIS
        kcfg_DarkenOuterCells3D->setText(tr2i18n("Darken 3-D cells outside the selected volume", 0));
        Q_UNUSED(ConfigGame);
    } // retranslateUi

};

namespace Ui {
    class ConfigGame: public Ui_ConfigGame {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGGAME_H

